# coding: utf-8

import logging
import re
import socket
import ConfigParser

from subprocess import Popen, PIPE

from tionix_client.helpers.updates import get_last_used_pypi
from tionix_client.helpers.i18 import ugettext as _

from novaclient.exceptions import ClientException as NovaClientException

try:
    from nova import version as openstack_version
except ImportError:
    openstack_version = None

logger = logging.getLogger(__name__)

NOVA_CONFIG = u'/etc/nova/nova.conf'
DB_SECTION = u'database'
DB_ITEM = u'connection'

OPENSTACK_VER_MAP = {'12': 'Liberty', '13': 'Mitaka'}


def get_hosts(nova_client, host_type, only_enabled=True):
    hosts = []
    if nova_client:
        try:
            services = nova_client.services.list(binary=host_type)
        except NovaClientException as err:
            logger.error(u'Unable to get nova hosts: {0}'.format(err))
            return hosts
        if only_enabled:
            hosts = [service.host for service in services
                     if unicode(service.status).lower() == u'enabled']
        else:
            hosts = [service.host for service in services]
    return hosts


def get_openstack_version():
    if openstack_version:
        version = openstack_version.version_string()
    else:
        logger.debug(u"Couldn't get OpenStack version")
        version = None
    return version


def get_db_version():
    """Parse nova.conf to get DB name and return DB version"""

    parser = ConfigParser.ConfigParser()
    version = None

    db_name = 'mysql'

    if parser.read(NOVA_CONFIG):
        try:
            db_str = parser.get(DB_SECTION, DB_ITEM)
        except (ConfigParser.NoSectionError, ConfigParser.NoOptionError):
            pass
        else:
            if db_str:
                split_pattern = '\+|\:'
                try:
                    db_name = re.split(split_pattern, db_str)[0]
                except re.error:
                    pass

    try:
        process = Popen(
            args=[db_name, '--version'],
            stdout=PIPE,
            stderr=PIPE
        )
        out, err = process.communicate()
        if not err:
            version = out.strip()
    except OSError:
        pass

    if not version:
        logger.debug(u"Couldn't get DB version")

    return version


def get_openstack_series(version):
    openstack_series = None
    series_re = re.compile('(?P<major>\d+).+(?P<minor>\d+).+(?P<other>\d+)')
    result = series_re.match(version)
    parsed_version = result.groupdict() if result else None
    if parsed_version:
        openstack_series = OPENSTACK_VER_MAP.get(parsed_version.get('major'))
    return openstack_series or ''


def get_system_info(nova_client=None):
    openstack_info = get_openstack_version() or u'-'
    openstack_series = get_openstack_series(openstack_info)
    hostname = socket.gethostname()
    compute_info = len(get_hosts(nova_client, u'nova-compute')) or u'-'
    conductor_info = len(get_hosts(nova_client, u'nova-conductor')) or u'-'
    db_info = get_db_version() or u'-'

    info = (
        (_(u'OpenStack version'), '{0} ({1})'.format(openstack_series,
                                                     openstack_info)),
        (_(u'Hostname'), hostname),
        (_(u'Conductor nodes'), conductor_info),
        (_(u'Compute nodes'), compute_info),
        (_(u'Database'), db_info),
        (_(u'Using PyPI'), get_last_used_pypi() or u'N\\A')
    )
    return info
