# coding: utf-8
# pylint: disable=no-name-in-module,import-error

import logging
import os

from distutils.version import LooseVersion

from tionix_client.helpers import PACKAGES
from tionix_client.helpers.memoized import memoized

import requests

from lxml import html

logger = logging.getLogger(__name__)


AUTOSETUP_CONF_DIR = u'/etc/tionix/autosetup/'
PYPI_LIST_FILE = os.path.join(AUTOSETUP_CONF_DIR, u'pypi_list.txt')


def get_last_used_pypi():
    host = None
    try:
        with open(PYPI_LIST_FILE, 'r') as f:
            last_line = f.readlines()[-1]
            last_line = last_line.rstrip('\n')
            parsed = last_line.split(';')
            if len(parsed) == 2:
                host = parsed[1]
    except Exception as err:
        logger.error(u'Unable to read PYPI list file: {0}'
                     u''.format(unicode(err)))
    return host


def get_latest_packages(pypi_host, packages):

    err_msg = u'Unable to get packages list from PYPI'
    versions = {package: None for package in packages}
    if pypi_host:
        pypi_index = 'http://{0}/packages/'.format(pypi_host)
        try:
            response = requests.get(pypi_index)
        except requests.ConnectionError as err:
            logger.error(u'{0}: {1}'.format(err_msg, unicode(err)))
        else:
            status_code = response.status_code
            if status_code == 200:
                html_tree = html.fromstring(response.content)
                for link in html_tree.xpath('//a/text()'):
                    package, version = link.split('-')
                    package = package.replace('_', '-')
                    if package in packages:
                        versions[package] = version.replace('.tar.gz', '')
            else:
                logger.error(u'{0}. Host returns status {1}.'
                             u''.format(err_msg, status_code))
    return versions


def get_actual_versions(packages):
    pypi_host = get_last_used_pypi()
    return get_latest_packages(pypi_host, packages)


def check_updates(installed_packages, actual=None):
    packages = {package.replace('_', '-'): version
                for package, version in installed_packages.items()}
    actual_versions = actual or get_actual_versions(packages.keys())
    updates = {}
    for package, version in packages.items():
        actual = actual_versions.get(package)
        installed = version
        if installed and actual:
            if LooseVersion(str(actual)) > LooseVersion(str(installed)):
                updates[package] = actual
        else:
            updates[package] = actual
    return updates


def get_latest_package_version(pip_name):
    packages_versions = get_all_latest_packages_versions()
    return packages_versions.get(pip_name)


@memoized
def get_all_latest_packages_versions():
    return get_actual_versions(PACKAGES.pip_names.values())
