# coding: utf-8

from novaclient import base
from novaclient.exceptions import NotFound


class ModuleResource(base.Resource):
    HUMAN_ID = True

    _module_name = 'tnx_module'
    _version = None

    def __init__(self, manager, info, loaded=False, resp=None):
        super(ModuleResource, self).__init__(manager, info, loaded, resp)
        self._version = info['version']

    @property
    def name(self):
        return self._module_name

    @name.setter
    def name(self, value):
        self._module_name = value

    def __repr__(self):
        return '<{0}: {1}>'.format(self._module_name, self._version)

    @property
    def version(self):
        return self._version


class ModuleManager(base.ManagerWithFind):
    resource_class = ModuleResource
    alias = 'tnx-modules'

    modules = [
        'tnx-cloud-control',
        'tnx-node-control',
        'tnx-monitor',
    ]

    # pylint: disable=arguments-differ
    def list(self, detailed=True, search_opts=None, marker=None, limit=None,
             sort_keys=None, sort_dirs=None):
        info_list = []
        for module in self.modules:
            try:
                info_list.append(self.get(module))
            except NotFound:
                continue
        return info_list

    def get(self, module):
        info = self._get(
            '/{0}'.format(module),
            'info'
        )
        info.name = module
        return info

    def get_version(self, module):
        info = self._get(
            '/{0}'.format(module),
            'info'
        )
        return info.version


class ModuleClientExtension(object):
    name = 'tnx_modules'
    manager_class = ModuleManager
