# coding: utf-8

import logging
import functools

from novaclient.exceptions import Unauthorized

from tionix_client import settings
from tionix_client.os_clients.keystone_client import get_keystone_client

logger = logging.getLogger(__name__)


class ResourceProxy(object):

    def __init__(self, client_proxy, name):
        self.resource = name
        self.client_proxy = client_proxy

    @property
    def client(self):
        return self.client_proxy.client

    @staticmethod
    def wrapped(func, client_proxy, resource, attr):

        @functools.wraps(func)
        def wrapper(*args, **kwargs):
            try:
                result = func(*args, **kwargs)
            except Unauthorized as _e:
                logger.warning(unicode(client_proxy.warning))
                client_proxy.refresh_client()

                res = getattr(client_proxy.client, resource)
                new_func = getattr(res, attr)
                result = new_func(*args, **kwargs)
            return result

        return wrapper

    def __getattr__(self, name):
        resource = getattr(self.client, self.resource)

        attr = getattr(resource, name)
        if callable(attr):
            return self.wrapped(attr, self.client_proxy, self.resource, name)
        return attr


class ClientProxy(object):

    def __init__(self, token, tenant_id):
        self.token = token
        self.tenant_id = tenant_id
        self.client = self.init_client()
        self.resources = {}
        self._create_trust()

    @property
    def warning(self):
        return ('Token expired for "{0}" user and recreated '
                'with "{1}" user trust token.'.format(self.trustee_name,
                                                      self.trustor_name))

    def _create_trust(self):
        trustor_client = get_keystone_client(token=self.token,
                                             tenant_id=self.tenant_id)
        trustor_id = trustor_client.auth_ref.user_id
        self.trustor_name = trustor_client.auth_ref.username

        trustee_user_client = get_keystone_client(
            username=settings.KEYSTONE['auth_user'],
            password=settings.KEYSTONE['auth_password'],
            tenant=settings.KEYSTONE['auth_tenant'])

        trustee_id = trustee_user_client.auth_ref.user_id
        self.trustee_name = trustee_user_client.auth_ref.username

        roles = trustor_client.auth_ref.role_names
        project = trustor_client.auth_ref.project_id
        trust = trustor_client.trusts.create(trustor_user=trustor_id,
                                             trustee_user=trustee_id,
                                             impersonation=True,
                                             role_names=roles,
                                             project=project)
        self.trust_id = trust.id

    def __getattr__(self, name):
        if name in self.resources:
            resource = self.resources[name]
        else:
            resource = ResourceProxy(self, name)
            self.resources[name] = resource

        return resource
