# coding: utf-8

import logging

from ceilometerclient.client import get_client

from tionix_client.os_clients.keystone_client import get_keystone_client
from tionix_client.settings import CEILOMETER_VERSION, KEYSTONE


logger = logging.getLogger(__name__)


# pylint:disable=too-many-arguments
def get_ceilometer_client(
        token=None, username=None, password=None, tenant=None, tenant_id=None,
        project_domain_name=None, user_domain_name=None, trust_id=None,
        auth_url=None, auth_version=None, timeout=None, keystone_client=None):
    """Get an authenticated Ceilometer client.

    :param token: auth token
    :param username: auth name
    :param password: auth password
    :param tenant: tenant name
    :param tenant_id: tenant id
    :param trust_id: Trust ID for trust scoping
    :param project_domain_name: project_domain_name
    :param user_domain_name: user_domain_name
    :param auth_url: auth_url
    :param auth_version: auth_version
    :param timeout: timeout
    :param keystone_client: keystone_client
    """

    if not keystone_client:
        keystone_client = get_keystone_client(
            token=token,
            username=username,
            password=password,
            tenant=tenant,
            tenant_id=tenant_id,
            user_domain_name=user_domain_name,
            project_domain_name=project_domain_name,
            auth_url=auth_url,
            auth_version=auth_version,
            trust_id=trust_id,
        )

    ceilometer_url = keystone_client.service_catalog.url_for(
        service_type=KEYSTONE['ceilometer_service_name'])

    try:
        ceilometer_client = get_client(
            CEILOMETER_VERSION,
            os_auth_token=keystone_client.auth_token,
            os_auth_url=keystone_client.auth_url,
            os_endpoint=ceilometer_url,
            timeout=timeout)
    except Exception as err:
        logger.error(u'Unable to get Ceilometer client: {0}'.format(err))
        raise err

    return ceilometer_client
