# coding: utf-8
# pylint:disable=too-many-arguments

import logging

from cinderclient.client import Client as CinderClient

from tionix_client import settings
from tionix_client.os_clients.keystone_client import get_keystone_client

logger = logging.getLogger(__name__)


def get_cinder_client(token=None, username=None, password=None,
                      tenant=None, tenant_id=None, user_domain_name=None,
                      project_domain_name=None, keystone_client=None):
    """Function to get cinder client by provided params.

    :param token: auth token
    :param username: auth name
    :param password: auth password
    :param tenant: tenant name
    :param tenant_id: tenant id
    :param user_domain_name: user_domain_name
    :param project_domain_name: project_domain_name
    :param keystone_client: keystone_client
    """
    if not keystone_client:
        keystone_client = get_keystone_client(
            token=token, username=username, password=password,
            tenant=tenant, tenant_id=tenant_id,
            user_domain_name=user_domain_name,
            project_domain_name=project_domain_name)

    cinder_url = keystone_client.service_catalog.url_for(
        service_type=settings.KEYSTONE['volume_service_name'])

    try:
        cinder_client = CinderClient(
            settings.CINDER_VERSION,
            username=username,
            api_key=keystone_client.auth_token,
            project_id=keystone_client.tenant_id,
            auth_url=cinder_url,
            auth_token=keystone_client.auth_token,
            bypass_url=cinder_url)

        # FIXME: openstack kilo, client not set token and management url
        cinder_client.client.auth_token = keystone_client.auth_token
        cinder_client.client.management_url = cinder_url

        # FIXME: openstack kilo, bug cinder client
        # not token in auth request headers
        # if not cinder_client.authenticate():
        #     raise AttributeError(u'Authorization error')
    except Exception as err:
        logger.error(unicode(err))
        raise err

    return cinder_client
