# coding: utf-8

"""
Power Device interface.
"""

from six.moves.urllib import parse

from novaclient import base


class PowerDevice(base.Resource):
    HUMAN_ID = True

    def __repr__(self):
        return '<PowerDevice: %s>' % self.name

    def delete(self):
        """
        Delete this power device.
        """
        self.manager.delete(self)

    def update(self, host=None, port_number=None, login=None, password=None):
        """
        Update this power device.
        """
        self.manager.update(self, host, port_number, login, password)

    def assigned_nodes(self):
        return self.manager.assigned_nodes(self)


class PowerDeviceManager(base.ManagerWithFind):
    resource_class = PowerDevice
    is_alphanum_id_allowed = True
    alias = 'tnx-nc-power-devices'
    collection_name = 'power_devices'
    member_name = 'power_device'
    member_alias = 'power-device'

    def list(self, detailed=True):
        qparams = {}

        return self._list(
            '/{0}{1}{2}'.format(
                self.alias,
                '/detail' if detailed else '',
                '?{0}'.format(parse.urlencode(qparams) if qparams else '')),
            self.collection_name
        )

    def get(self, identifier):
        return self._get(
            '/{0}/{1}'.format(
                self.alias, base.getid(identifier)), self.member_name)

    def delete(self, identifier):
        self._delete('/%s/%s' % (self.alias, base.getid(identifier)))

    def _build_body(self, **kwargs):
        return {
            self.member_alias: {k: v for k, v in kwargs.iteritems() if v}
        }

    def create(self, **kwargs):

        body = self._build_body(**kwargs)

        return self._create('/{0}'.format(self.alias), body, self.member_name)

    def update(self, identifier, **kwargs):

        body = self._build_body(**kwargs)

        return self._update(
            '/{0}/{1}'.format(self.alias, base.getid(identifier)),
            body,
            self.member_name)

    def types(self):
        _resp, body = self.api.client.get(
            u'/{0}/types'.format(self.alias)
        )
        return body['types']

    def protocols(self):
        _resp, body = self.api.client.get(
            u'/{0}/protocols'.format(self.alias)
        )
        return body['protocols']

    def assigned_nodes(self, identifier):
        resp, body = self._action('assigned-nodes', base.getid(identifier))
        return body['result']

    def _action(self, action, identifier, body=None):
        url = '/{0}/{1}/action'.format(self.alias, identifier)
        return self.api.client.post(url, body={action: body or {}})


class PowerDeviceNovaCliExt(object):
    name = 'power_devices'
    manager_class = PowerDeviceManager
