# coding: utf-8

from urllib import urlencode

from novaclient import base


class SessionResource(base.Resource):
    HUMAN_ID = True

    _module_name = 'tnx_sessions'
    _version = None


class SessionManager(base.ManagerWithFind):
    resource_class = SessionResource
    alias = 'tnx-sessions'

    def _build_list_request(self, user_id=None, instance_id=None,
                            domain_id=None):
        qparams = {}

        if user_id:
            qparams['user_id'] = user_id
        if instance_id:
            qparams['instance_id'] = instance_id
        if domain_id:
            qparams['domain_id'] = domain_id

        return '/{0}{1}'.format(
            self.alias,
            '?{0}'.format(urlencode(qparams, doseq=True))
        )

    def create(self, **kwargs):
        """
        POST method to create session
        """
        body = {
            'session-info': kwargs
        }
        url = '/{0}/add_session'.format(self.alias)
        return self.api.client.post(url, body=body)

    def list(self, user_id=None, instance_id=None, domain_id=None):

        return self._list(
            self._build_list_request(user_id=user_id, instance_id=instance_id,
                                     domain_id=domain_id),
            'sessions')


class SessionClientExtension(object):
    name = 'tnx_sessions'
    manager_class = SessionManager
