# coding: utf-8

import logging.config
import os

from collections import defaultdict
from tionix_client.settings_handler import (
    update_global_from_settings, update_module_settings)

BASE_DIR = os.path.dirname(os.path.abspath(__file__))

NOVA_VERSION = 2.25

CINDER_VERSION = 2

CEILOMETER_VERSION = 2

GLANCE_VERSION = 2

NEUTRON_VERSION = 2

KEYSTONE = {
    'auth_url': 'http://localhost:5000',
    'auth_version': '3',
    'auth_user': 'admin',
    'auth_password': 'admin',
    'auth_tenant': 'admin',
    'compute_service_name': 'compute',
    'image_service_name': 'image',
    'network_service_name': 'network',
    'volume_service_name': 'volumev2',
    'ceilometer_service_name': 'metering',
    'user_domain_name': 'default',
    'project_domain_name': 'default'
}

LOG_LEVEL = 'INFO'

logger = logging.getLogger()
logger.setLevel(LOG_LEVEL)

config_path = '/etc/tionix/tionix.yaml'
opened_file = None

try:
    opened_file = open(config_path)
except IOError as e:
    logger.warning(u'{0}. Settings set to default.'.format(unicode(e)))
else:
    module_settings = defaultdict(dict)
    gl = globals()
    update_module_settings(module_settings, gl, opened_file)
    # Have to update log level before updating other variables for
    # proper logger output
    if module_settings['LOG_LEVEL']:
        logger.setLevel(module_settings['LOG_LEVEL'])

    update_global_from_settings(module_settings, gl)
    logger.setLevel(LOG_LEVEL)
finally:
    if opened_file:
        opened_file.close()
